//
//  FlutterFimSegmentviewController.m
//  flutter_fim_segmentview
//
//  Created by zhuyuhui on 2020/5/29.
//

#import "FlutterFimSegmentviewController.h"
#import <JKCategories/JKCategories.h>
#import <JXCategoryView/JXCategoryView.h>
#import <MJExtension/MJExtension.h>
#import "FlutterFimSegmentviewArgs.h"

@interface FlutterFimSegmentViewController()<JXCategoryViewDelegate>

@end

@implementation FlutterFimSegmentViewController{
    int64_t _viewId;
    FlutterMethodChannel* _channel;
    UIActivityIndicatorView * _indicator;
    JXCategoryTitleView * _categoryView;
    FlutterFimSegmentviewArgs * _argModel;
}

- (instancetype)initWithWithFrame:(CGRect)frame viewIdentifier:(int64_t)viewId arguments:(id)args binaryMessenger:(NSObject<FlutterBinaryMessenger> *)messenger{
    if ([super init]) {
        _viewId = viewId;
        _argModel = [FlutterFimSegmentviewArgs mj_objectWithKeyValues:args];
        if (!_argModel.titles) {
            _argModel.titles = @[];
        }
        if (!_argModel.titleNumberOfLines) {
            _argModel.titleNumberOfLines = @"1";
        }
        if (!_argModel.titleColor) {
            _argModel.titleColor = @"000000";
        }
        if (!_argModel.titleSelectedColor) {
            _argModel.titleSelectedColor = @"FF0000";
        }
        if (!_argModel.titleFont) {
            _argModel.titleFont = @"15";
        }
        if (!_argModel.titleSelectedFont) {
            _argModel.titleSelectedFont = @"16";
        }
        if (!_argModel.indicatorColor) {
            _argModel.indicatorColor = @"FF0000";
        }
        
        _categoryView = [[JXCategoryTitleView alloc] initWithFrame:frame];
        _categoryView.delegate = self;
        _categoryView.titles = _argModel.titles;
        _categoryView.titleNumberOfLines = _argModel.titleNumberOfLines.integerValue;
        _categoryView.titleFont = [UIFont systemFontOfSize:_argModel.titleFont.floatValue];
        _categoryView.titleSelectedFont = [UIFont systemFontOfSize:_argModel.titleSelectedFont.floatValue];
        _categoryView.titleColor = [UIColor jk_colorWithHexString:_argModel.titleColor];
        _categoryView.titleSelectedColor = [UIColor jk_colorWithHexString:_argModel.titleSelectedColor];
        _categoryView.defaultSelectedIndex = _argModel.defaultSelectedIndex;
        _categoryView.titleColorGradientEnabled = YES;
        
        JXCategoryIndicatorLineView *lineView = [[JXCategoryIndicatorLineView alloc] init];
        lineView.indicatorColor = [UIColor jk_colorWithHexString:_argModel.indicatorColor];
        lineView.indicatorWidth = JXCategoryViewAutomaticDimension;
        _categoryView.indicators = @[lineView];
        
        
        NSString* channelName = [NSString stringWithFormat:@"plugins/flutter_fim_segmentview_%lld", viewId];
        _channel = [FlutterMethodChannel methodChannelWithName:channelName binaryMessenger:messenger];
        __weak __typeof__(self) weakSelf = self;
        [_channel setMethodCallHandler:^(FlutterMethodCall *  call, FlutterResult  result) {
            [weakSelf onMethodCall:call result:result];
        }];
        
        [_channel invokeMethod:@"didClickSelectedItemAtIndex" arguments:@{@"index" : @(_argModel.defaultSelectedIndex)}];
    }
    
    return self;
}

-(UIView *)view{
    return _categoryView;
}

-(void)onMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result{
    if ([[call method] isEqualToString:@"resetDatas"]) {
        NSDictionary *dic = call.arguments;
        if (dic && [dic isKindOfClass:[NSDictionary class]]) {
            NSArray <NSString *>*titles = dic[@"titles"];
            NSUInteger selectedIndex = [dic[@"selectIndex"] integerValue];
            _argModel.titles = titles;
            _categoryView.titles = titles;
            [_categoryView reloadData];
            [_categoryView selectItemAtIndex:selectedIndex];
        }
        [_indicator startAnimating];
    } else if ([[call method] isEqualToString:@"stop"]){
        [_indicator stopAnimating];
    } else {
        result(FlutterMethodNotImplemented);
    }
}

#pragma mark - JXCategoryViewDelegate

- (void)categoryView:(JXCategoryBaseView *)categoryView didClickSelectedItemAtIndex:(NSInteger)index {
    [_channel invokeMethod:@"didClickSelectedItemAtIndex" arguments:@{@"index" : @(index)}];
}


@end

@implementation FlutterFimSegmentViewFactory{
    NSObject<FlutterBinaryMessenger>*_messenger;
}

- (instancetype)initWithMessenger:(NSObject<FlutterBinaryMessenger> *)messager{
    self = [super init];
    if (self) {
        _messenger = messager;
    }
    return self;
}

-(NSObject<FlutterMessageCodec> *)createArgsCodec{
    return [FlutterStandardMessageCodec sharedInstance];
}

-(NSObject<FlutterPlatformView> *)createWithFrame:(CGRect)frame viewIdentifier:(int64_t)viewId arguments:(id)args{
    
   FlutterFimSegmentViewController*activity = [[FlutterFimSegmentViewController alloc] initWithWithFrame:frame viewIdentifier:viewId arguments:args binaryMessenger:_messenger];
    return activity;
    
}
@end

