import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

///先定义一个controller创建成功回调 后面会用到
typedef void ViewCreatedCallback(MethodChannelSegmentViewPlatform controller);

typedef void DidClickSelectedItemAtIndex(int index);

class MethodChannelSegmentViewPlatform {
  final _PlatformCallbacksHandler _platformCallbacksHandler;
  final MethodChannel _channel;

  MethodChannelSegmentViewPlatform(int id, this._platformCallbacksHandler)
      : assert(_platformCallbacksHandler != null),
        _channel = MethodChannel('plugins/flutter_fim_segmentview_$id') {
    _channel.setMethodCallHandler(_onMethodCall);
  }

  Future<void> resetDatas(List datas, int selectIndex) async {
    return _channel.invokeMethod('resetDatas', {
      "titles": datas,
      "selectIndex": selectIndex,
    });
  }

  Future<bool> _onMethodCall(MethodCall call) async {
    switch (call.method) {
      case 'didClickSelectedItemAtIndex':
        final int index = call.arguments['index'];
        _platformCallbacksHandler.didClickSelectedItemAtIndex(index);
        return true;
    }
    throw MissingPluginException(
      '${call.method} was invoked but has no handler',
    );
  }
}

class FlutterFimSegmentView extends StatefulWidget {
  final ViewCreatedCallback onViewCreated;
  final DidClickSelectedItemAtIndex didClickSelectedItemAtIndex;

  final Map<String, dynamic> arguments;

  const FlutterFimSegmentView(
      {Key key,
      this.onViewCreated,
      this.arguments,
      this.didClickSelectedItemAtIndex})
      : super(key: key);

  @override
  _FlutterFimSegmentViewState createState() => _FlutterFimSegmentViewState();
}

class _FlutterFimSegmentViewState extends State<FlutterFimSegmentView> {
  _PlatformCallbacksHandler _platformCallbacksHandler;

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    if (defaultTargetPlatform == TargetPlatform.iOS) {
      ///加载iOS原生视图
      return UiKitView(
        viewType: "plugins/flutter_fim_segmentview",

        ///视图标识符 要和原生 保持一致 要不然加载不到视图
        onPlatformViewCreated: _onPlatformViewCreated,

        ///原生视图创建成功的回调
        creationParams: widget.arguments,

        /// 用来编码 creationParams 的形式，可选 [StandardMessageCodec], [JSONMessageCodec], [StringCodec], or [BinaryCodec]
        /// 如果存在 creationParams，则该值不能为null
        creationParamsCodec: new StandardMessageCodec(),
      );
    }
    return Text('不支持的平台 $defaultTargetPlatform ');
  }

  ///我也是刚学Flutter 所以我理解的:这个基本上是固定写法
  void _onPlatformViewCreated(int id) {
    if (widget.onViewCreated == null) {
      return;
    }
    widget.onViewCreated(
        MethodChannelSegmentViewPlatform(id, _platformCallbacksHandler));
  }

  @override
  void initState() {
    super.initState();
    _platformCallbacksHandler = _PlatformCallbacksHandler(widget);
  }

  @override
  void didUpdateWidget(FlutterFimSegmentView oldWidget) {
    super.didUpdateWidget(oldWidget);
    _platformCallbacksHandler._widget = widget;
  }
}

class _PlatformCallbacksHandler {
  FlutterFimSegmentView _widget;

  _PlatformCallbacksHandler(this._widget);

  void didClickSelectedItemAtIndex(int index) {
    if (_widget.didClickSelectedItemAtIndex != null) {
      _widget.didClickSelectedItemAtIndex(index);
    }
  }
}
